@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,20 +15,23 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#define VERSION "BRLTTY driver for TSI displays, version 2.72 (January 2003)"
-#define COPYRIGHT "Copyright (C) 1996-2003 by Stphane Doyon " \
+#define VERSION "BRLTTY driver for TSI displays, version 2.74 (April 2004)"
+#define COPYRIGHT "Copyright (C) 1996-2004 by Stphane Doyon " \
                   "<s.doyon@videotron.ca>"
 /* TSI/braille.c - Braille display driver for TSI displays
  *
  * Written by Stphane Doyon (s.doyon@videotron.ca)
  *
  * It attempts full support for Navigator 20/40/80 and Powerbraille 40/65/80.
- * It is designed to be compiled into BRLTTY version 3.2.
+ * It is designed to be compiled into BRLTTY version 3.5.
  *
  * History:
+ * Version 2.74 apr2004: use message() to report low battery condition.
+ * Version 2.73 jan2004: Fix key bindings for speech commands for PB80.
+ *   Add CMD_SPKHOME to help.
  * Version 2.72 jan2003: brl->buffer now allocated by core.
- * Version 2.71: Added CMD_LEARN, CMD_NXPROMPT/CMD_PRPROMPT and CMD_SIXDOTS.
- * Version 2.70: Added CR_CUTAPPEND, CR_CUTLINE, CR_SETMARK, CR_GOTOMARK
+ * Version 2.71: Added CMD_LEARN, BRL_CMD_NXPROMPT/CMD_PRPROMPT and CMD_SIXDOTS.
+ * Version 2.70: Added CR_CUTAPPEND, BRL_BLK_CUTLINE, BRL_BLK_SETMARK, BRL_BLK_GOTOMARK
  *   and CR_SETLEFT. Changed binding for NXSEARCH.. Adjusted PB80 cut&paste
  *   bindings. Replaced CMD_CUT_BEG/CMD_CUT_END by CR_CUTBEGIN/CR_CUTRECT,
  *   and CMD_CSRJMP by CR_ROUTE+0. Adjusted cut_cursor for new cut&paste
@@ -83,22 +86,21 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <sys/termios.h>
-#include <sys/ioctl.h>
 #include <string.h>
 
-#include "Programs/brl.h"
 #include "Programs/misc.h"
+#include "Programs/message.h"
 
 #define BRLSTAT ST_PB80Style
 #include "Programs/brl_driver.h"
 #include "braille.h"
+#include "Programs/serial.h"
 
 /* Braille display parameters that do not change */
 #define BRLROWS 1		/* only one row on braille display */
 
 /* Type of delay the display requires after sending it commands.
-   0 -> no delay, 1 -> tcdrain only, 2 -> tcdrain + wait for SEND_DELAY. */
+   0 -> no delay, 1 -> drain only, 2 -> drain + wait for SEND_DELAY. */
 static int slow_update;
 
 /* Whether multiple packets can be sent for a single update. */
@@ -132,31 +134,15 @@ static struct timeval lastcmd_time, last_readbrl_time;
 static struct timezone dum_tz;
 /* Those functions it is OK to repeat */
 static int repeat_list[] =
-{CMD_FWINRT, CMD_FWINLT, CMD_LNUP, CMD_LNDN, CMD_WINUP, CMD_WINDN,
- CMD_CHRLT, CMD_CHRRT, VAL_PASSKEY+VPK_CURSOR_LEFT,
- VAL_PASSKEY+VPK_CURSOR_RIGHT, VAL_PASSKEY+VPK_CURSOR_UP,
- VAL_PASSKEY+VPK_CURSOR_DOWN,
- CMD_CSRTRK, 0};
-
-/* Low battery warning */
-#ifdef LOW_BATTERY_WARN
-/* We must code the dots forming the message, not its text since handling
-   of translation tables has been moved to the main module of brltty. */
-static unsigned char battery_msg_20[] =		/* "-><- Low batteries" */
-  {0x30, 0x1a, 0x25, 0x30, 0x00, 0x55, 0x19, 0x2e, 0x00, 0x05,
-     0x01, 0x1e, 0x1e, 0x09, 0x1d, 0x06, 0x09, 0x16, 0x00, 0x00},
-          battery_msg_40[] =	/* "-><-- Navigator:  Batteries are low" */
-  {0x30, 0x1a, 0x25, 0x30, 0x30, 0x00, 0x5b, 0x01, 0x35, 0x06,
-     0x0f, 0x01, 0x1e, 0x19, 0x1d, 0x29, 0x00, 0x00, 0x45, 0x01,
-     0x1e, 0x1e, 0x09, 0x1d, 0x06, 0x09, 0x16, 0x00, 0x01, 0x1d,
-     0x09, 0x00, 0x15, 0x19, 0x2e, 0x00, 0x00, 0x00, 0x00, 0x00};
-#endif /* LOW_BATTERY_WARN */
+{BRL_CMD_FWINRT, BRL_CMD_FWINLT, BRL_CMD_LNUP, BRL_CMD_LNDN, BRL_CMD_WINUP, BRL_CMD_WINDN,
+ BRL_CMD_CHRLT, BRL_CMD_CHRRT, BRL_BLK_PASSKEY+BRL_KEY_CURSOR_LEFT,
+ BRL_BLK_PASSKEY+BRL_KEY_CURSOR_RIGHT, BRL_BLK_PASSKEY+BRL_KEY_CURSOR_UP,
+ BRL_BLK_PASSKEY+BRL_KEY_CURSOR_DOWN,
+ BRL_CMD_CSRTRK, 0};
 
 /* This defines the mapping between brltty and Navigator's dots coding. */
 static TranslationTable outputTable;
 
-/* delay between auto-detect attemps at initialization */
-#define DETECT_DELAY (2000)	/* msec */
 /* Stabilization delay after changing baud rate */
 #define BAUD_DELAY (100)
 
@@ -304,9 +290,7 @@ static struct inbytedesc pb_key_desc[PB_KEY_LEN] =
 
 /* Global variables */
 
-static int brl_fd;              /* file descriptor for comm port */
-static struct termios oldtio,	/* old terminal settings for com port */
-                      curtio;   /* current settings */
+static SerialDevice *serialDevice;              /* file descriptor for comm port */
 static unsigned char *rawdata,	/* translated data to send to display */
                      *prevdata, /* previous data sent */
                      *dispbuf;
@@ -318,9 +302,6 @@ static unsigned char has_sw,    /* flag: has routing keys or not */
 				   being 0). On PB80 this excludes the 81st. */
                      sw_bcnt;   /* bytes of sensor switch information */
 static char disp_ver[Q_VER_LENGTH]; /* version of the hardware */
-#ifdef LOW_BATTERY_WARN
-static unsigned char *battery_msg;     /* low battery warning msg */
-#endif /* LOW_BATTERY_WARN */
 
 static enum { NAV20_40=0, NAV80, PB40, PB65_80} displayType;
 
@@ -333,51 +314,20 @@ brl_identify (void)
 
 
 static int
-CheckCTSLine(int fd)
-/* If supported (might not be portable) check that the CTS line (RS232)
-   is up. */
+myread(void *buf, unsigned len)
 {
-#if defined(CHECKCTS) && defined(TIOCMGET) && defined(TIOCM_CTS)
-  int flags;
-  if (ioctl(fd, TIOCMGET, &flags) < 0){
-    LogError("TIOCMGET");
-    return(-1);
-  }else if(flags & TIOCM_CTS)
-    return(1);
-  else return(0);
-#else /* defined(CHECKCTS) && defined(TIOCMGET) && defined(TIOCM_CTS) */
-  return(1);
-#endif /* defined(CHECKCTS) && defined(TIOCMGET) && defined(TIOCM_CTS) */
-}
-
-static int
-myread(int fd, void *buf, unsigned len)
-/* This is a replacement for read for use in nonblocking mode: when
-   c_cc[VTIME] = 1, c_cc[VMIN] = 0. We want to read len bytes into buf, but
-   return early if the timeout expires. I would have thought setting
-   c_cc[VMIN] to len would have done the trick, but apparently c_cc[VMIN]>0
-   means to wait indefinitly for at least 1byte, which we don't want. */
-{
-  char *ptr = (char *)buf;
-  int r, l = 0;
-  while(l < len){
-    r = read(fd,ptr+l,len-l);
-    if(r == 0) return(l);
-    else if(r<0) return(r);
-    l += r;
-  }
-  return(l);
+  return serialReadData(serialDevice,buf,len,100,100);
 }
 
 
 static int
-QueryDisplay(int brl_fd, char *reply)
+QueryDisplay(char *reply)
 /* For auto-detect: send query, read response and validate response header. */
 {
-  if (write(brl_fd, BRL_QUERY, DIM_BRL_QUERY) == DIM_BRL_QUERY) {
-    if (awaitInput(brl_fd, 100)) {
-      int count;
-      if ((count = myread(brl_fd, reply, Q_REPLY_LENGTH)) != -1) {
+  int count;
+  if ((count = serialWriteData(serialDevice, BRL_QUERY, DIM_BRL_QUERY)) == DIM_BRL_QUERY) {
+    if (serialAwaitInput(serialDevice, 100)) {
+      if ((count = myread(reply, Q_REPLY_LENGTH)) != -1) {
         if ((count == Q_REPLY_LENGTH) && (memcmp(reply, Q_HEADER, Q_HEADER_LENGTH) == 0)) {
           LogPrint(LOG_DEBUG, "Valid reply received.");
           return 1;
@@ -385,11 +335,13 @@ QueryDisplay(int brl_fd, char *reply)
           LogBytes("Unexpected response", reply, count);
         }
       } else {
-        LogError("read");
+        LogError("Read");
       }
     }
+  } else if (count != -1) {
+    LogPrint(LOG_ERR, "Short write: %d < %d", count, DIM_BRL_QUERY);
   } else {
-    LogError("write");
+    LogError("Write");
   }
   return 0;
 }
@@ -401,13 +353,13 @@ ResetTypematic (void)
 {
   static unsigned char params[2] =
     {BRL_TYPEMATIC_DELAY, BRL_TYPEMATIC_REPEAT};
-  write (brl_fd, BRL_TYPEMATIC, DIM_BRL_TYPEMATIC);
-  write (brl_fd, &params, 2);
+  serialWriteData (serialDevice, BRL_TYPEMATIC, DIM_BRL_TYPEMATIC);
+  serialWriteData (serialDevice, &params, 2);
 }
 
 
 static int
-brl_open (BrailleDisplay *brl, char **parameters, const char *tty)
+brl_open (BrailleDisplay *brl, char **parameters, const char *device)
 {
   int i=0;
   char reply[Q_REPLY_LENGTH];
@@ -418,66 +370,31 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *tty)
     makeOutputTable(&dots, &outputTable);
   }
 
+  if (!isSerialDevice(&device)) {
+    unsupportedDevice(device);
+    return 0;
+  }
+
   rawdata = prevdata = NULL;
 
   /* Open the Braille display device for random access */
-  if (!openSerialDevice(tty, &brl_fd, &oldtio)) goto failure;
-
-  /* Construct new settings by working from current state */
-  memcpy(&curtio, &oldtio, sizeof(struct termios));
-  rawSerialDevice(&curtio);
-  /* local */
-  curtio.c_lflag &= ~TOSTOP; /* no SIGTTOU to backgrounded processes */
-  /* control */
-  curtio.c_cflag |= CLOCAL /* ignore status lines (carrier detect...) */
-                  | CREAD; /* enable reading */
-  curtio.c_cflag &= ~( CSTOPB /* 1 stop bit */
-		      | CRTSCTS /* disable hardware flow control */
-                      | PARENB /* disable parity generation and detection */
-                     );
-  /* input */
-  curtio.c_iflag &= ~( INPCK /* no input parity check */
-                      | IXOFF /* don't send XON/XOFF */
-		     );
-
-  /* noncanonical: for first operation */
-  curtio.c_cc[VTIME] = 1;  /* 0.1sec timeout between chars on input */
-  curtio.c_cc[VMIN] = 0; /* no minimum input. it would have been nice to set
-			    this to Q_REPLY_LENGTH except it waits
-			    indefinitly for the first char if VTIME != 0... */
-
+  if (!(serialDevice = serialOpenDevice(device))) goto failure;
   /* Try to detect display by sending query */
-  while(1){
-    /* Check RS232 wires: we should have CTS if the device is connected and
-       ON. The NAV40 lowers CTS when off, the PB80 doesn't, and I don't know
-       about the others. */
-    if((i = CheckCTSLine(brl_fd)) == -1)
-      goto failure;
-    else if(!i){
-      LogPrint(LOG_INFO, "Display is off or not connected");
-      while(!i){
-	delay(DETECT_DELAY);
-	if((i = CheckCTSLine(brl_fd)) == -1)
-	  goto failure;
-      }
-      LogPrint(LOG_DEBUG,"Device was connected or activated");
-    }
-    LogPrint(LOG_DEBUG,"Sending query at 9600bps");
-    if(!resetSerialDevice(brl_fd, &curtio, B9600)) goto failure;
-    if(QueryDisplay(brl_fd,reply)) break;
+  LogPrint(LOG_DEBUG,"Sending query at 9600bps");
+  if(!serialRestartDevice(serialDevice, 9600)) goto failure;
+  if(!QueryDisplay(reply)){
 #ifdef HIGHBAUD
     /* Then send the query at 19200bps, in case a PB was left ON
        at that speed */
     LogPrint(LOG_DEBUG,"Sending query at 19200bps");
-    if(!setSerialDevice(brl_fd, &curtio, B19200)) goto failure;
-    if(QueryDisplay(brl_fd,reply)) break;
+    if(!serialSetBaud(serialDevice, 19200)) goto failure;
+    if(!QueryDisplay(reply)) goto failure;
 #endif /* HIGHBAUD */
-    delay(DETECT_DELAY);
   }
 
   memcpy (disp_ver, &reply[Q_OFFSET_VER], Q_VER_LENGTH);
   ncells = reply[Q_OFFSET_COLS];
-  LogPrint(LOG_INFO,"Display replyed: %d cells, version %c%c%c%c", ncells,
+  LogPrint(LOG_INFO,"Display replied: %d cells, version %c%c%c%c", ncells,
 	   disp_ver[0], disp_ver[1], disp_ver[2], disp_ver[3]);
 
   brl_cols = ncells;
@@ -557,29 +474,22 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *tty)
 #endif /* NO_MULTIPLE_UPDATES */
   if(slow_update == 2) no_multiple_updates = 1;
 
-#ifdef LOW_BATTERY_WARN
-  if (brl_cols == 20){
-      battery_msg = battery_msg_20;
-    }else{
-      battery_msg = battery_msg_40;
-    }
-#endif /* LOW_BATTERY_WARN */
-
 #ifdef HIGHBAUD
   if(speed == 2){ /* if supported (PB) go to 19.2Kbps */
-    write (brl_fd, BRL_UART192, DIM_BRL_UART192);
-    tcdrain(brl_fd);
-    delay(BAUD_DELAY);
-    if(!setSerialDevice(brl_fd, &curtio, B19200)) goto failure;
+    serialWriteData (serialDevice, BRL_UART192, DIM_BRL_UART192);
+    serialDrainOutput(serialDevice);
+    approximateDelay(BAUD_DELAY);
+    if(!serialSetBaud(serialDevice, 19200)) goto failure;
     LogPrint(LOG_DEBUG,"Switched to 19200bps. Checking if display followed.");
-    if(QueryDisplay(brl_fd,reply))
+    if(QueryDisplay(reply))
       LogPrint(LOG_DEBUG,"Display responded at 19200bps.");
     else{
       LogPrint(LOG_INFO,"Display did not respond at 19200bps, "
 	       "falling back to 9600bps.");
-      if(!setSerialDevice(brl_fd, &curtio, B9600)) goto failure;
-      delay(BAUD_DELAY); /* just to be safe */
-      if(QueryDisplay(brl_fd,reply)) {
+      if(!serialSetBaud(serialDevice, 9600)) goto failure;
+      serialDrainOutput(serialDevice);
+      approximateDelay(BAUD_DELAY); /* just to be safe */
+      if(QueryDisplay(reply)) {
 	LogPrint(LOG_INFO,"Found display again at 9600bps.");
 	LogPrint(LOG_INFO, "Must be a TSI emulator.");
       }else{
@@ -623,7 +533,6 @@ brl_open (BrailleDisplay *brl, char **parameters, const char *tty)
   return 1;
 
 failure:;
-  LogPrint(LOG_WARNING,"TSI driver giving up");
   brl_close(brl);
   return 0;
 }
@@ -632,9 +541,8 @@ failure:;
 static void 
 brl_close (BrailleDisplay *brl)
 {
-  if (brl_fd >= 0) {
-    tcsetattr (brl_fd, TCSADRAIN, &oldtio);
-    close (brl_fd);
+  if (serialDevice) {
+    serialCloseDevice (serialDevice);
   }
   free (dispbuf);
   free (rawdata);
@@ -672,7 +580,7 @@ display (const unsigned char *pattern,
      (with long lines) holding down the up/down arrow key on the PC keyboard.
 
      pb40 has no problems: it apparently can take whatever we throw at
-     it. Nav40 is good but we tcdrain just to be safe.
+     it. Nav40 is good but we drain just to be safe.
 
      pb80 (twice larger but twice as fast as nav40) cannot take a continuous
      full speed flow. There is no flow control: apparently not supported
@@ -690,14 +598,14 @@ display (const unsigned char *pattern,
      I no longer have access to a Nav40 and PB80 for testing: I only have a
      PB40.  */
 
-  write (brl_fd, rawdata, DIM_BRL_SEND + 2 * length);
+  serialWriteData (serialDevice, rawdata, DIM_BRL_SEND + 2 * length);
 
-  /* First a tcdrain after the write helps make sure we don't fill up the
+  /* First a drain after the write helps make sure we don't fill up the
      buffer with info that will be overwritten immediately. This is not needed
      on pb40, but it might be good on nav40 (which runs only at 9600bps).
 
      Then for pb80 (and probably also needed for nav80 though untested) as
-     well as for some TSI emulators we add a supplementary delay: tcdrain
+     well as for some TSI emulators we add a supplementary delay: drain
      probably waits until the bytes are in the UART, but we still need a few
      ms to send them out.  This delay will combine with the sleep for this
      BRLTTY cycle (delay in the main program loop of 40ms).  Keeping the delay
@@ -705,17 +613,17 @@ display (const unsigned char *pattern,
      gives the display time to show what we're sending.
 
      I found experimentally that a delay of 30ms seems best (at least for
-     pb80). A longer delay, or using delay instead of shortdelay is too
-     much. This makes the pb a little bit sluggish, but prevents any
-     communication problems.  */
+     pb80). A longer delay, or using approximateDelay() instead of
+     accurateDelay() is too much. This makes the pb a little bit sluggish,
+     but prevents any communication problems. */
 
   switch(slow_update){
   /* 0 does nothing */
   case 1: /* nav 40 */
-    tcdrain(brl_fd); break;
+    serialDrainOutput(serialDevice); break;
   case 2: /* nav80, pb80, some emulators */
-    tcdrain(brl_fd);
-    shortdelay(SEND_DELAY);
+    serialDrainOutput(serialDevice);
+    accurateDelay(SEND_DELAY);
     break;
   };
 }
@@ -819,11 +727,11 @@ flicker ()
       memset (buf, FLICKER_CHAR, ncells);
 
       display_all (buf);
-      shortdelay (FLICKER_DELAY);
+      accurateDelay (FLICKER_DELAY);
       display_all (prevdata);
-      shortdelay (FLICKER_DELAY);
+      accurateDelay (FLICKER_DELAY);
       display_all (buf);
-      shortdelay (FLICKER_DELAY);
+      accurateDelay (FLICKER_DELAY);
       /* Note that we don't put prevdata back on the display, since flicker()
          normally preceeds the displaying of a special message. */
 
@@ -832,18 +740,6 @@ flicker ()
 }
 
 
-#ifdef LOW_BATTERY_WARN
-static void 
-do_battery_warn ()
-{
-  flicker ();
-  display_all (battery_msg);
-  delay (BATTERY_DELAY);
-  memset(prevdata, 255, ncells); /* force refreshing of the display */
-}
-#endif /* LOW_BATTERY_WARN */
-
-
 /* OK this one is pretty strange and ugly. readbrl() reads keys from
    the display's key pads. It calls cut_cursor() if it gets a certain key
    press. cut_cursor() is an elaborate function that allows selection of
@@ -899,41 +795,41 @@ cut_cursor (BrailleDisplay *brl)
       display_all (prevdata);
       prevdata[pos] = oldchar;
 
-      while ((key = brl_readCommand (brl, CMDS_SCREEN)) == EOF) delay(1); /* just yield */
-      if((key &VAL_BLK_MASK) == CR_CUTBEGIN)
-	  res = CR_CUTBEGIN + pos;
-      else if((key &VAL_BLK_MASK) == CR_CUTAPPEND)
-	  res = CR_CUTAPPEND + pos;
-      else if((key &VAL_BLK_MASK) == CR_CUTRECT) {
-	  res = CR_CUTRECT + pos;
+      while ((key = brl_readCommand (brl, BRL_CTX_SCREEN)) == EOF) approximateDelay(1); /* just yield */
+      if((key &BRL_MSK_BLK) == BRL_BLK_CUTBEGIN)
+	  res = BRL_BLK_CUTBEGIN + pos;
+      else if((key &BRL_MSK_BLK) == BRL_BLK_CUTAPPEND)
+	  res = BRL_BLK_CUTAPPEND + pos;
+      else if((key &BRL_MSK_BLK) == BRL_BLK_CUTRECT) {
+	  res = BRL_BLK_CUTRECT + pos;
 	  pos = -1;
-      }else if((key &VAL_BLK_MASK) == CR_CUTLINE) {
-	  res = CR_CUTLINE + pos;
+      }else if((key &BRL_MSK_BLK) == BRL_BLK_CUTLINE) {
+	  res = BRL_BLK_CUTLINE + pos;
 	  pos = -1;
       }else switch (key)
 	{
-	case CMD_FWINRT:
+	case BRL_CMD_FWINRT:
 	  pos++;
 	  break;
-	case CMD_FWINLT:
+	case BRL_CMD_FWINLT:
 	  pos--;
 	  break;
-	case CMD_LNUP:
+	case BRL_CMD_LNUP:
 	  pos += 5;
 	  break;
-	case CMD_LNDN:
+	case BRL_CMD_LNDN:
 	  pos -= 5;
 	  break;
-	case VAL_PASSKEY+VPK_CURSOR_RIGHT:
+	case BRL_BLK_PASSKEY+BRL_KEY_CURSOR_RIGHT:
 	  pos = brl_cols - 1;
 	  break;
-	case VAL_PASSKEY+VPK_CURSOR_LEFT:
+	case BRL_BLK_PASSKEY+BRL_KEY_CURSOR_LEFT:
 	  pos = 0;
 	  break;
-	case VAL_PASSKEY+VPK_CURSOR_UP:
+	case BRL_BLK_PASSKEY+BRL_KEY_CURSOR_UP:
 	  pos += 10;
 	  break;
-	case VAL_PASSKEY+VPK_CURSOR_DOWN:
+	case BRL_BLK_PASSKEY+BRL_KEY_CURSOR_DOWN:
 	  pos -= 10;
 	  break;
 	case CMD_CUT_CURSOR:
@@ -963,8 +859,8 @@ cut_cursor (BrailleDisplay *brl)
 #define KEYSPECIAL(code,action) \
     case code: { action }; break;
 #define KEYSW(codeon, codeoff, result) \
-    case codeon: res = result | VAL_SWITCHON; break; \
-    case codeoff: res = result | VAL_SWITCHOFF; break;
+    case codeon: res = result | BRL_FLG_TOGGLE_ON; break; \
+    case codeoff: res = result | BRL_FLG_TOGGLE_OFF; break;
 
 /* For cursor routing */
 /* lookup so find out if a certain key is active */
@@ -987,7 +883,7 @@ cut_cursor (BrailleDisplay *brl)
 #define MAXREAD 10
 
 static int 
-brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
+brl_readCommand (BrailleDisplay *brl, BRL_DriverCommandContext context)
 {
   /* static bit vector recording currently pressed sensor switches (for
      repetition detection) */
@@ -1009,7 +905,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
   int skip_this_cmd = 0;
 
   gettimeofday (&now, &dum_tz);
-  if (elapsedMilliseconds(&last_readbrl_time, &now) > READBRL_SKIP_TIME)
+  if (millisecondsBetween(&last_readbrl_time, &now) > READBRL_SKIP_TIME)
     /* if the key we get this time is the same as the one we returned at last
        call, and if it has been abnormally long since we were called
        (presumably sound was being played or a message displayed) then
@@ -1039,28 +935,23 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
    any keys in a certain time. That a 2byte header + 10 more bytes ignored.
  */
 
-  /* reset to nonblocking */
-  curtio.c_cc[VTIME] = 0;
-  curtio.c_cc[VMIN] = 0;
-  tcsetattr (brl_fd, TCSANOW, &curtio);
   /* Check for first byte */
-  if (read (brl_fd, buf, 1) < 1){
-    if((i = elapsedMilliseconds(&last_ping, &now) > PING_INTRVL)){
-      int ping_due = (pings==0 || (elapsedMilliseconds(&last_ping_sent, &now)
+  if (serialReadData (serialDevice, buf, 1, 0, 0) != 1){
+    if((i = millisecondsBetween(&last_ping, &now) > PING_INTRVL)){
+      int ping_due = (pings==0 || (millisecondsBetween(&last_ping_sent, &now)
 				   > PING_REPLY_DELAY));
-      if((pings>=PING_MAXNQUERY && ping_due)
-	 || CheckCTSLine(brl_fd) != 1)
-	return CMD_RESTARTBRL;
+      if((pings>=PING_MAXNQUERY && ping_due))
+	return BRL_CMD_RESTARTBRL;
       else if(ping_due){
 	LogPrint(LOG_DEBUG,"Display idle: sending query");
-	tcdrain(brl_fd);
-	delay(2*SEND_DELAY);
-	write (brl_fd, BRL_QUERY, DIM_BRL_QUERY);
+	serialDrainOutput(serialDevice);
+	approximateDelay(2*SEND_DELAY);
+	serialWriteData (serialDevice, BRL_QUERY, DIM_BRL_QUERY);
 	if(slow_update == 1)
-	  tcdrain(brl_fd);
+	  serialDrainOutput(serialDevice);
 	else if(slow_update == 2){
-	  tcdrain(brl_fd);
-	  delay(SEND_DELAY);
+	  serialDrainOutput(serialDevice);
+	  approximateDelay(SEND_DELAY);
 	}
 	pings++;
 	gettimeofday(&last_ping_sent, &dum_tz);
@@ -1072,12 +963,8 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
   gettimeofday(&last_ping, &dum_tz);
   pings=0;
 
-  /* further reads will wait a bit to get a complete sequence */
-  curtio.c_cc[VTIME] = 1;
-  curtio.c_cc[VMIN] = 0;
-  tcsetattr (brl_fd, TCSANOW, &curtio);
 #ifdef RECV_DELAY
-  shortdelay(SEND_DELAY);
+  accurateDelay(SEND_DELAY);
 #endif /* RECV_DELAY */
 
   /* read bytes */
@@ -1109,7 +996,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
       }
     }
     i++;
-    if (!myread (brl_fd, buf+i, 1))
+    if (myread (buf+i, 1) != 1)
       return (EOF);
   }/* while */
 
@@ -1122,12 +1009,14 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
   }
 
   if(packtype == K_BATTERY){
-    do_battery_warn ();
+    const char *text = "-><- Display battery is low";
+    if(brl_cols < strlen(text)) text = "Display battery low";
+    message(text, MSG_WAITKEY);
     return (EOF);
   }else if(packtype == K_QUERYREP){
     /* flush the last 10bytes of the reply. */
     LogPrint(LOG_DEBUG,"Got reply to idle ping");
-    myread(brl_fd, buf, Q_REPLY_LENGTH - Q_HEADER_LENGTH);
+    myread(buf, Q_REPLY_LENGTH - Q_HEADER_LENGTH);
     return(EOF);
   }else if(packtype == K_NAVKEY || packtype == K_PBKEY){
     /* construct code */
@@ -1152,7 +1041,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 
     /* read next byte: it indicates length of sequence */
     /* still VMIN = 0, VTIME = 1 */
-    if (!myread (brl_fd, &cnt, 1))
+    if (myread (&cnt, 1) != 1)
       return (EOF);
     
     /* if sw_bcnt and cnt disagree, then must be garbage??? */
@@ -1160,25 +1049,13 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
     if (cnt != sw_bcnt)
       return (EOF);
 
-    /* skip the vertical keys info */
-#if 0
-    curtio.c_cc[VTIME] = 1;
-    curtio.c_cc[VMIN] = SW_NVERT;
-    tcsetattr (brl_fd, TCSANOW, &curtio);
-#endif /* 0 */
-    if (myread (brl_fd, buf, SW_NVERT) != SW_NVERT)
+    if (myread (buf, SW_NVERT) != SW_NVERT)
       return (EOF);
     cnt -= SW_NVERT;
     /* cnt now gives the number of bytes describing horizontal
        routing keys only */
     
-    /* Finally, the horizontal keys */
-#if 0
-    curtio.c_cc[VTIME] = 1;
-    curtio.c_cc[VMIN] = cnt;
-    tcsetattr (brl_fd, TCSANOW, &curtio);
-#endif /* 0 */
-    if (myread (brl_fd, sw_stat, cnt) != cnt)
+    if (myread (sw_stat, cnt) != cnt)
       return (EOF);
 
     /* if key press is maintained, then packet is resent by display
@@ -1209,70 +1086,70 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
     ignore_routing = 1;
     if(sw_howmany == 1){
       switch(code){
-	KEYAND(KEY_BUT3) KEY(KEY_BRIGHT, CR_CUTBEGIN + sw_which[0]);
-	KEYAND(KEY_BUT2) KEY(KEY_BLEFT, CR_CUTRECT + sw_which[0]);
-	KEYAND(KEY_R2DN) KEY (KEY_BDOWN, CR_NXINDENT + sw_which[0]);
-	KEYAND(KEY_R2UP) KEY (KEY_BUP, CR_PRINDENT + sw_which[0]);
-	KEY (KEY_CROUND, CR_SETMARK + sw_which[0]);
-	KEYAND(KEY_CNCV) KEY (KEY_BROUND, CR_GOTOMARK + sw_which[0]);
-	KEY (KEY_CUP, CR_SETLEFT + sw_which[0]);
-	KEY (KEY_CDOWN, CR_SWITCHVT + sw_which[0]);
+	KEYAND(KEY_BUT3) KEY(KEY_BRIGHT, BRL_BLK_CUTBEGIN + sw_which[0]);
+	KEYAND(KEY_BUT2) KEY(KEY_BLEFT, BRL_BLK_CUTRECT + sw_which[0]);
+	KEYAND(KEY_R2DN) KEY (KEY_BDOWN, BRL_BLK_NXINDENT + sw_which[0]);
+	KEYAND(KEY_R2UP) KEY (KEY_BUP, BRL_BLK_PRINDENT + sw_which[0]);
+	KEY (KEY_CROUND, BRL_BLK_SETMARK + sw_which[0]);
+	KEYAND(KEY_CNCV) KEY (KEY_BROUND, BRL_BLK_GOTOMARK + sw_which[0]);
+	KEY (KEY_CUP, BRL_BLK_SETLEFT + sw_which[0]);
+	KEY (KEY_CDOWN, BRL_BLK_SWITCHVT + sw_which[0]);
 	KEYAND(KEY_CDOWN | KEY_BUP) KEY(KEY_CUP | KEY_CDOWN,
-					CR_DESCCHAR +sw_which[0]);
+					BRL_BLK_DESCCHAR +sw_which[0]);
       }
     }else if(sw_howmany == 2) {
       if(sw_which[0]+1 == sw_which[1]
 	 && (code == KEY_BRIGHT || code == KEY_BUT3))
-	res = CR_CUTAPPEND + sw_which[0];
+	res = BRL_BLK_CUTAPPEND + sw_which[0];
       else if(sw_which[0]+1 == sw_which[1]
 	 && (code == KEY_BLEFT || code == KEY_BUT2))
-	res = CR_CUTLINE + sw_which[1];
+	res = BRL_BLK_CUTLINE + sw_which[1];
       else if(sw_which[0]==0 && sw_which[1]==1){
 	switch(code){
-	  KEYAND(KEY_R2DN) KEY (KEY_BDOWN, CMD_NXPGRPH);
-	  KEYAND(KEY_R2UP) KEY (KEY_BUP, CMD_PRPGRPH);
+	  KEYAND(KEY_R2DN) KEY (KEY_BDOWN, BRL_CMD_NXPGRPH);
+	  KEYAND(KEY_R2UP) KEY (KEY_BUP, BRL_CMD_PRPGRPH);
 	}
       }else if(sw_which[0]==1 && sw_which[1]==2){
 	switch(code){
-	  KEY (KEY_BDOWN, CMD_NXPROMPT);
-	  KEY (KEY_BUP, CMD_PRPROMPT);
+	  KEY (KEY_BDOWN, BRL_CMD_NXPROMPT);
+	  KEY (KEY_BUP, BRL_CMD_PRPROMPT);
 	}
       }else if(sw_which[0]==0 && sw_which[1]==2){
 	switch(code){
-	  KEYAND(KEY_R2DN) KEY (KEY_BDOWN, CMD_NXSEARCH);
-	  KEYAND(KEY_R2UP) KEY (KEY_BUP, CMD_PRSEARCH);
+	  KEYAND(KEY_R2DN) KEY (KEY_BDOWN, BRL_CMD_NXSEARCH);
+	  KEYAND(KEY_R2UP) KEY (KEY_BUP, BRL_CMD_PRSEARCH);
 	}
       }
     }
   }else if (has_sw && sw_howmany)	/* routing key */
     {
       if (sw_howmany == 1)
-	res = CR_ROUTE + sw_which[0];
+	res = BRL_BLK_ROUTE + sw_which[0];
 #if 0
      else if (sw_howmany == 3 && sw_which[1] == sw_lastkey - 1
 	       && sw_which[2] == sw_lastkey)
-	res = CR_CUTBEGIN + sw_which[0];
+	res = BRL_BLK_CUTBEGIN + sw_which[0];
       else if (sw_howmany == 3 && sw_which[0] == 0 && sw_which[1] == 1)
- 	res = CR_CUTRECT + sw_which[2];
+ 	res = BRL_BLK_CUTRECT + sw_which[2];
 #endif /* 0 */
       else if(sw_howmany == 2 && sw_which[0] == 1 && sw_which[1] == 2)
- 	res = CMD_PASTE;
+ 	res = BRL_CMD_PASTE;
       else if (sw_howmany == 2 && sw_which[0] == 0 && sw_which[1] == 1)
-	res = CMD_CHRLT;
+	res = BRL_CMD_CHRLT;
       else if (sw_howmany == 2 && sw_which[0] == sw_lastkey - 1
 	       && sw_which[1] == sw_lastkey)
-	res = CMD_CHRRT;
+	res = BRL_CMD_CHRRT;
       else if (sw_howmany == 2 && sw_which[0] == 0 && sw_which[1] == 2)
-	res = CMD_HWINLT;
+	res = BRL_CMD_HWINLT;
       else if (sw_howmany == 2 && sw_which[0] == sw_lastkey - 2
 	       && sw_which[1] == sw_lastkey)
-	res = CMD_HWINRT;
+	res = BRL_CMD_HWINRT;
       else if (sw_howmany == 2 && sw_which[0] == 0
 	       && sw_which[1] == sw_lastkey)
-	res = CMD_HELP;
+	res = BRL_CMD_HELP;
       else if (sw_howmany == 4 && sw_which[0] == 0 && sw_which[1] == 1
 	       && sw_which[2] == sw_lastkey-1 && sw_which[3] == sw_lastkey)
-	res = CMD_LEARN;
+	res = BRL_CMD_LEARN;
 #if 0
       else if (sw_howmany == 2 && sw_which[0] == 1
 	       && sw_which[1] == sw_lastkey - 1)
@@ -1283,121 +1160,124 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
 	}
 #endif /* 0 */
       else if(sw_howmany == 3 && sw_which[0]+2 == sw_which[1]){
-	  res = CR_CUTBEGIN + sw_which[0];
-	  pending_cmd = CR_CUTRECT + sw_which[2];
+	  res = BRL_BLK_CUTBEGIN + sw_which[0];
+	  pending_cmd = BRL_BLK_CUTRECT + sw_which[2];
 	}
     }
   else switch (code){
   /* renames: CLEFT=BUT1 CRIGHT=BUT2 CUP=R1UP CDOWN=R2DN CROUNT=CNVX */
 
   /* movement */
-    KEYAND(KEY_BAR1) KEYAND(KEY_R2UP) KEY (KEY_BUP, CMD_LNUP);
+    KEYAND(KEY_BAR1) KEYAND(KEY_R2UP) KEY (KEY_BUP, BRL_CMD_LNUP);
     KEYAND(KEY_BAR2) KEYAND(KEY_BAR3) KEYAND(KEY_BAR4)
-      KEYAND(KEY_R2DN) KEY (KEY_BDOWN, CMD_LNDN);
-    KEYAND(KEY_BUT3) KEY (KEY_BLEFT, CMD_FWINLT);
-    KEYAND(KEY_BUT4) KEY (KEY_BRIGHT, CMD_FWINRT);
+      KEYAND(KEY_R2DN) KEY (KEY_BDOWN, BRL_CMD_LNDN);
+    KEYAND(KEY_BUT3) KEY (KEY_BLEFT, BRL_CMD_FWINLT);
+    KEYAND(KEY_BUT4) KEY (KEY_BRIGHT, BRL_CMD_FWINRT);
 
-    KEYAND(KEY_CNCV) KEY (KEY_BROUND, CMD_HOME);
-    KEYAND(KEY_CNCV | KEY_CUP) KEY(KEY_BROUND | KEY_CUP, CMD_BACK);
-    KEY (KEY_CROUND, (cmds == CMDS_PREFS) ? CMD_MENU_PREV_SETTING
-	 : CMD_CSRTRK);
+    KEYAND(KEY_CNCV) KEY (KEY_BROUND, BRL_CMD_HOME);
+    KEYAND(KEY_CNCV | KEY_CUP) KEY(KEY_BROUND | KEY_CUP, BRL_CMD_BACK);
+    KEY (KEY_CROUND, (context == BRL_CTX_PREFS) ? BRL_CMD_MENU_PREV_SETTING
+	 : BRL_CMD_CSRTRK);
 
-    KEYAND(KEY_BUT1 | KEY_BAR1) KEY (KEY_BLEFT | KEY_BUP, CMD_TOP_LEFT);
-    KEYAND(KEY_BUT1 | KEY_BAR2) KEY (KEY_BLEFT | KEY_BDOWN, CMD_BOT_LEFT);
+    KEYAND(KEY_BUT1 | KEY_BAR1) KEY (KEY_BLEFT | KEY_BUP, BRL_CMD_TOP_LEFT);
+    KEYAND(KEY_BUT1 | KEY_BAR2) KEY (KEY_BLEFT | KEY_BDOWN, BRL_CMD_BOT_LEFT);
 
-    KEYAND(KEY_BUT2 | KEY_BAR1) KEY (KEY_BROUND | KEY_BUP, CMD_PRDIFLN);
+    KEYAND(KEY_BUT2 | KEY_BAR1) KEY (KEY_BROUND | KEY_BUP, BRL_CMD_PRDIFLN);
     KEYAND(KEY_BUT2 | KEY_BAR2) KEYAND(KEY_BUT2 | KEY_BAR3)
-      KEYAND(KEY_BUT2 | KEY_BAR4) KEY (KEY_BROUND | KEY_BDOWN, CMD_NXDIFLN);
-    KEYAND(KEY_BUT2 | KEY_R2UP) KEY(KEY_CROUND | KEY_BUP, CMD_ATTRUP);
-    KEYAND(KEY_BUT2 | KEY_R2DN) KEY(KEY_CROUND | KEY_BDOWN, CMD_ATTRDN);
+      KEYAND(KEY_BUT2 | KEY_BAR4) KEY (KEY_BROUND | KEY_BDOWN, BRL_CMD_NXDIFLN);
+    KEYAND(KEY_BUT2 | KEY_R2UP) KEY(KEY_CROUND | KEY_BUP, BRL_CMD_ATTRUP);
+    KEYAND(KEY_BUT2 | KEY_R2DN) KEY(KEY_CROUND | KEY_BDOWN, BRL_CMD_ATTRDN);
 
-    KEY (KEY_CLEFT | KEY_CROUND, CMD_CHRLT);
-    KEY (KEY_CRIGHT | KEY_CROUND, CMD_CHRRT);
+    KEY (KEY_CLEFT | KEY_CROUND, BRL_CMD_CHRLT);
+    KEY (KEY_CRIGHT | KEY_CROUND, BRL_CMD_CHRRT);
 
-    KEY (KEY_CLEFT | KEY_CUP, CMD_HWINLT);
-    KEY (KEY_CRIGHT | KEY_CUP, CMD_HWINRT);
+    KEY (KEY_CLEFT | KEY_CUP, BRL_CMD_HWINLT);
+    KEY (KEY_CRIGHT | KEY_CUP, BRL_CMD_HWINRT);
 
-    KEYAND(KEY_BUT1|KEY_BUT2|KEY_BAR1)  KEY (KEY_CROUND | KEY_CUP, CMD_WINUP);
-    KEYAND(KEY_BUT1|KEY_BUT2|KEY_BAR2) KEY (KEY_CROUND | KEY_CDOWN, CMD_WINDN);
+    KEYAND(KEY_BUT1|KEY_BUT2|KEY_BAR1)  KEY (KEY_CROUND | KEY_CUP, BRL_CMD_WINUP);
+    KEYAND(KEY_BUT1|KEY_BUT2|KEY_BAR2) KEY (KEY_CROUND | KEY_CDOWN, BRL_CMD_WINDN);
 
-    KEYAND(KEY_R1UP | KEY_BUT3) KEY(KEY_CUP | KEY_BLEFT, CMD_LNBEG);
-    KEYAND(KEY_R1UP | KEY_BUT4) KEY(KEY_CUP | KEY_BRIGHT, CMD_LNEND);
+    KEYAND(KEY_R1UP | KEY_BUT3) KEY(KEY_CUP | KEY_BLEFT, BRL_CMD_LNBEG);
+    KEYAND(KEY_R1UP | KEY_BUT4) KEY(KEY_CUP | KEY_BRIGHT, BRL_CMD_LNEND);
 
   /* keyboard cursor keys simulation */
-    KEY (KEY_CLEFT, VAL_PASSKEY+VPK_CURSOR_LEFT);
-    KEY (KEY_CRIGHT, VAL_PASSKEY+VPK_CURSOR_RIGHT);
-    KEY (KEY_CUP, (cmds == CMDS_PREFS && displayType == PB40)
-	 ? CMD_MENU_PREV_SETTING : VAL_PASSKEY+VPK_CURSOR_UP);
-    KEY (KEY_CDOWN, (cmds == CMDS_PREFS && displayType == PB40)
-	 ? CMD_MENU_NEXT_SETTING : VAL_PASSKEY+VPK_CURSOR_DOWN);
+    KEY (KEY_CLEFT, BRL_BLK_PASSKEY+BRL_KEY_CURSOR_LEFT);
+    KEY (KEY_CRIGHT, BRL_BLK_PASSKEY+BRL_KEY_CURSOR_RIGHT);
+    KEY (KEY_CUP, (context == BRL_CTX_PREFS && displayType == PB40)
+	 ? BRL_CMD_MENU_PREV_SETTING : BRL_BLK_PASSKEY+BRL_KEY_CURSOR_UP);
+    KEY (KEY_CDOWN, (context == BRL_CTX_PREFS && displayType == PB40)
+	 ? BRL_CMD_MENU_NEXT_SETTING : BRL_BLK_PASSKEY+BRL_KEY_CURSOR_DOWN);
 
   /* special modes */
-    KEY (KEY_CLEFT | KEY_CRIGHT, CMD_HELP);
+    KEY (KEY_CLEFT | KEY_CRIGHT, BRL_CMD_HELP);
     KEYAND (KEY_BUT1 | KEY_BUT2 | KEY_BUT3 | KEY_BUT4)
-      KEY (KEY_CLEFT | KEY_CRIGHT | KEY_CUP | KEY_CDOWN, CMD_LEARN);
-    KEY (KEY_CROUND | KEY_BROUND, CMD_FREEZE);
+      KEY (KEY_CLEFT | KEY_CRIGHT | KEY_CUP | KEY_CDOWN, BRL_CMD_LEARN);
+    KEY (KEY_CROUND | KEY_BROUND, BRL_CMD_FREEZE);
     KEYAND (KEY_CUP | KEY_BDOWN) KEYAND(KEY_BUT3 | KEY_BUT4)
-      KEY (KEY_BUP | KEY_BDOWN, CMD_INFO);
+      KEY (KEY_BUP | KEY_BDOWN, BRL_CMD_INFO);
     KEYAND (KEY_CDOWN | KEY_BUP)
-      KEY (KEY_CUP | KEY_CDOWN, CMD_ATTRVIS);
+      KEY (KEY_CUP | KEY_CDOWN, BRL_CMD_ATTRVIS);
     KEYAND (KEY_CDOWN | KEY_BUP | KEY_CROUND)
-      KEY (KEY_CUP | KEY_CDOWN | KEY_CROUND, CMD_DISPMD)
+      KEY (KEY_CUP | KEY_CDOWN | KEY_CROUND, BRL_CMD_DISPMD)
 
   /* Emulation of cursor routing */
-    KEYAND(KEY_R1DN | KEY_R2DN) KEY (KEY_CDOWN | KEY_BDOWN, CMD_CSRJMP_VERT);
-    KEY (KEY_CDOWN | KEY_BDOWN | KEY_BLEFT, CR_ROUTE +0);
-    KEY (KEY_CDOWN | KEY_BDOWN | KEY_BRIGHT, CR_ROUTE + 3 * brl_cols / 4 - 1);
+    KEYAND(KEY_R1DN | KEY_R2DN) KEY (KEY_CDOWN | KEY_BDOWN, BRL_CMD_CSRJMP_VERT);
+    KEY (KEY_CDOWN | KEY_BDOWN | KEY_BLEFT, BRL_BLK_ROUTE +0);
+    KEY (KEY_CDOWN | KEY_BDOWN | KEY_BRIGHT, BRL_BLK_ROUTE + 3 * brl_cols / 4 - 1);
 
   /* Emulation of routing keys for cut&paste */
-    KEY (KEY_CLEFT | KEY_BROUND, CR_CUTBEGIN +0);
-    KEY (KEY_CLEFT | KEY_BROUND | KEY_BUP, CR_CUTAPPEND +0);
-    KEY (KEY_CRIGHT | KEY_BROUND, CR_CUTRECT +brl_cols-1);
-    KEY (KEY_CRIGHT | KEY_BROUND | KEY_BUP, CR_CUTLINE +brl_cols-1);
+    KEY (KEY_CLEFT | KEY_BROUND, BRL_BLK_CUTBEGIN +0);
+    KEY (KEY_CLEFT | KEY_BROUND | KEY_BUP, BRL_BLK_CUTAPPEND +0);
+    KEY (KEY_CRIGHT | KEY_BROUND, BRL_BLK_CUTRECT +brl_cols-1);
+    KEY (KEY_CRIGHT | KEY_BROUND | KEY_BUP, BRL_BLK_CUTLINE +brl_cols-1);
     KEY (KEY_CLEFT | KEY_CRIGHT | KEY_BROUND, CMD_CUT_CURSOR);  /* special: see
 	 				    at the end of this fn */
   /* paste */
-    KEY (KEY_CDOWN | KEY_BROUND, CMD_PASTE);
+    KEY (KEY_CDOWN | KEY_BROUND, BRL_CMD_PASTE);
 
   /* speech */
   /* experimental speech support */
-    KEYAND(KEY_CRIGHT | KEY_BLEFT) KEY (KEY_BRIGHT | KEY_BDOWN, CMD_SAY_LINE);
-    KEY (KEY_BLEFT | KEY_BRIGHT | KEY_BDOWN, CMD_SAY_BELOW);
-    KEY (KEY_BROUND | KEY_BRIGHT, CMD_SPKHOME);
+    KEYAND(KEY_CRIGHT | KEY_BLEFT) KEYAND(KEY_BAR2 | KEY_R2DN)
+      KEY (KEY_BRIGHT | KEY_BDOWN, BRL_CMD_SAY_LINE);
+    KEYAND(KEY_BAR1 | KEY_BAR2 | KEY_R2DN) 
+      KEY (KEY_BLEFT | KEY_BRIGHT | KEY_BDOWN, BRL_CMD_SAY_BELOW);
+    KEYAND(KEY_BROUND | KEY_BAR2) KEY (KEY_BROUND | KEY_BRIGHT, BRL_CMD_SPKHOME);
     KEYAND(KEY_CRIGHT | KEY_CUP | KEY_BLEFT | KEY_BUP)
-      KEY (KEY_BRIGHT | KEY_BUP, CMD_MUTE);
-    KEY (KEY_BRIGHT | KEY_BUP | KEY_CUP | KEY_BLEFT, CMD_RESTARTSPEECH);
+      KEYAND(KEY_BAR2 | KEY_R2UP) KEY (KEY_BRIGHT | KEY_BUP, BRL_CMD_MUTE);
+    KEYAND(KEY_BAR1 | KEY_BAR2 | KEY_R1UP | KEY_R2UP)
+      KEY (KEY_BRIGHT | KEY_BUP | KEY_CUP | KEY_BLEFT, BRL_CMD_RESTARTSPEECH);
     
   /* preferences menu */
-    KEYAND(KEY_BAR1 | KEY_BAR2) KEY (KEY_BLEFT | KEY_BRIGHT, CMD_PREFMENU);
+    KEYAND(KEY_BAR1 | KEY_BAR2) KEY (KEY_BLEFT | KEY_BRIGHT, BRL_CMD_PREFMENU);
     KEYAND(KEY_BAR1 | KEY_BAR2 | KEY_CNCV) 
-      KEY (KEY_BLEFT | KEY_BRIGHT | KEY_BROUND, CMD_PREFSAVE);
+      KEY (KEY_BLEFT | KEY_BRIGHT | KEY_BROUND, BRL_CMD_PREFSAVE);
     KEYAND(KEY_BAR1 | KEY_BAR2 | KEY_CNVX | KEY_CNCV) 
-      KEY (KEY_CROUND | KEY_BLEFT | KEY_BRIGHT | KEY_BROUND, CMD_PREFLOAD);
-    KEY (KEY_BLEFT | KEY_BRIGHT | KEY_BROUND | KEY_BDOWN, CMD_SKPIDLNS);
-    KEY (KEY_BLEFT | KEY_BRIGHT | KEY_CROUND | KEY_BDOWN, CMD_SKPIDLNS);
-    KEY (KEY_CLEFT | KEY_BLEFT | KEY_BRIGHT, CMD_SLIDEWIN);
+      KEY (KEY_CROUND | KEY_BLEFT | KEY_BRIGHT | KEY_BROUND, BRL_CMD_PREFLOAD);
+    KEY (KEY_BLEFT | KEY_BRIGHT | KEY_BROUND | KEY_BDOWN, BRL_CMD_SKPIDLNS);
+    KEY (KEY_BLEFT | KEY_BRIGHT | KEY_CROUND | KEY_BDOWN, BRL_CMD_SKPIDLNS);
+    KEY (KEY_CLEFT | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_SLIDEWIN);
     KEYAND(KEY_BUT2 | KEY_BAR1 | KEY_BAR2)
-      KEY (KEY_CLEFT | KEY_CROUND | KEY_BLEFT | KEY_BRIGHT, CMD_TUNES);
+      KEY (KEY_CLEFT | KEY_CROUND | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_TUNES);
     KEYAND(KEY_BUT1 | KEY_BAR1 | KEY_BAR2)    
-      KEY (KEY_CUP | KEY_BLEFT | KEY_BRIGHT, CMD_CSRVIS);
+      KEY (KEY_CUP | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_CSRVIS);
     KEYAND(KEY_R1DN | KEY_BAR1 | KEY_BAR2)    
-      KEY (KEY_CDOWN | KEY_BLEFT | KEY_BRIGHT, CMD_SIXDOTS);
+      KEY (KEY_CDOWN | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_SIXDOTS);
     KEYAND(KEY_BUT1 | KEY_BAR1 | KEY_BAR2 | KEY_CNVX)
-      KEY (KEY_CROUND | KEY_CUP | KEY_BLEFT | KEY_BRIGHT, CMD_CSRBLINK);
+      KEY (KEY_CROUND | KEY_CUP | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_CSRBLINK);
     KEYAND(KEY_BUT2 | KEY_BAR1 | KEY_BAR2 | KEY_CNVX)
-      KEY (KEY_CROUND | KEY_CDOWN | KEY_BLEFT | KEY_BRIGHT, CMD_CAPBLINK);
+      KEY (KEY_CROUND | KEY_CDOWN | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_CAPBLINK);
     KEYAND (KEY_CROUND | KEY_BUP | KEY_BLEFT | KEY_BRIGHT)
-      KEY (KEY_CROUND | KEY_CRIGHT | KEY_BLEFT | KEY_BRIGHT, CMD_ATTRBLINK);
+      KEY (KEY_CROUND | KEY_CRIGHT | KEY_BLEFT | KEY_BRIGHT, BRL_CMD_ATTRBLINK);
 
   /* PB80 switches */
-    KEYSW(KEY_S1UP, KEY_S1DN, CMD_ATTRVIS);
+    KEYSW(KEY_S1UP, KEY_S1DN, BRL_CMD_ATTRVIS);
     KEYSW(KEY_S1UP |KEY_BAR1|KEY_BAR2|KEY_CNVX,
-	  KEY_S1DN |KEY_BAR1|KEY_BAR2|KEY_CNVX, CMD_ATTRBLINK);
-    KEYSW(KEY_S1UP | KEY_CNVX, KEY_S1DN | KEY_CNVX, CMD_ATTRBLINK);
-    KEYSW(KEY_S2UP, KEY_S2DN, CMD_FREEZE);
-    KEYSW(KEY_S3UP, KEY_S3DN, CMD_SKPIDLNS);
-    KEYSW(KEY_S3UP |KEY_BAR1, KEY_S3DN |KEY_BAR1, CMD_SKPIDLNS);
-    KEYSW(KEY_S4UP, KEY_S4DN, CMD_DISPMD);
+	  KEY_S1DN |KEY_BAR1|KEY_BAR2|KEY_CNVX, BRL_CMD_ATTRBLINK);
+    KEYSW(KEY_S1UP | KEY_CNVX, KEY_S1DN | KEY_CNVX, BRL_CMD_ATTRBLINK);
+    KEYSW(KEY_S2UP, KEY_S2DN, BRL_CMD_FREEZE);
+    KEYSW(KEY_S3UP, KEY_S3DN, BRL_CMD_SKPIDLNS);
+    KEYSW(KEY_S3UP |KEY_BAR1, KEY_S3DN |KEY_BAR1, BRL_CMD_SKPIDLNS);
+    KEYSW(KEY_S4UP, KEY_S4DN, BRL_CMD_DISPMD);
 
 #if 0
   /* typematic reset */
@@ -1415,7 +1295,7 @@ brl_readCommand (BrailleDisplay *brl, DriverCommandContext cmds)
     }else{
       /* if to short a time has elapsed since last command, ignore this one */
       gettimeofday (&now, &dum_tz);
-      if (elapsedMilliseconds(&lastcmd_time, &now) < NONREPEAT_TIMEOUT)
+      if (millisecondsBetween(&lastcmd_time, &now) < NONREPEAT_TIMEOUT)
 	res = EOF;
     }
   }
