@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the Linux console (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2003 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,6 +18,10 @@
 #ifndef _SCR_FROZEN_H
 #define _SCR_FROZEN_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include "scr_base.h"
 
 typedef struct {
@@ -28,4 +32,8 @@ typedef struct {
 
 extern void initializeFrozenScreen (FrozenScreen *);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* _SCR_FROZEN_H */
